function experiment_model_fitter_1

% (c) Ali Tariq, 2018.

clc
close all
clear all

n = 10;   % The number of generations the simulation is run for.

T = 10;   % The length of time a generation is left to grow for. We keep it
          % the same for all generations.

tspan = linspace(0,T);  % Time interval over which a solution is plotted.
                        % This creates 100 evenly spaced points between 0
                        % and T.

A_mat = zeros(length(tspan),n); % This is a zero matrix which will accept
                                % the output values for each generation of
                                % aphids over the time interval. Each
                                % column will store the data for each
                                % generation.

% We will set up a loop to solve the equation for the n generations of
% aphids we want to run the simulation for. However the first generation
% of aphids will be unaffected by the transgenic plants, meaning their
% birth and death rates will vary from subseqent generations. Therefore, we
% will run the simulation for the first generation spearately and create a
% loop for generations 2 to n.

A0 = 10;     % The initial number of aphids in the first generation.
b0 = 0.1;    % The birth rate for the first generation.
d0 = 0;      % The death rate for the first generation.

[~,Sol] = ode45(@(t,A)growthode(t,A,b0,d0), tspan, A0);  % Call solver.

A_mat(:,1) = Sol; % Replace the first column of A_mat with the data output
                  % from solving the equation for the first generation.

% Now we run a loop to add to A_mat the equivalent data for the remaining
% generations (2 to n). We assune that for generation 2 onwards the
% birth and death rate remain the same. Should you want them to change for
% each generation you must define this directly in the loop.

b = 0.05;   % Birth rate of generation 2 to n.
d = 0;    % Death rate of generation 2 to n.

A_ini = ( (b0*A0) / (b0-d0) )*((exp((b0-d0)*T)) - 1); % Initial value for 
                                                      % subsequent
                                                      % generations.
                                                      
                                                      

for i = 2:n  % the loop runs from generation 2 to n. i is the iterator.
    
    [~,Sol] = ode45(@(t,A)growthode(t,A,b,d), tspan, A_ini); % Call solver.
    
    A_mat(:,i) = Sol; % Append the ith solution to the ith column of A_mat.
    
    A_ini = ( (b*A_ini) / (b-d) )*((exp((b-d)*T)) - 1); % Redefine the
                                                        % initial
                                                        % condition.
end

% At this point we should now have a 100xn matrix, A_mat, whose ith column
% contains the population growth data for the ith generation over the
% growth period of time length, T. 

% Now we add some normally distributed noise to A_mat to cloud the data.
% Then we will use Matlab's curve fitter to try to obtain the value of b-d
% for each generation.

M_noise = 0.3*randn(length(tspan),n); % Create a matrix of normally 
                                      % distributed noise.

A_data = abs(A_mat + M_noise); % Add the noise to the solutions of the ODE 
                               % to create our synthetic data. Use absolute
                               % value since data is positive.

t_data = tspan; % Rename tspan (the time points the ODE was solved for) to
                % t_data for readability.
                               

Para_Mat = zeros(1,n); % Row vector to store the found values of b-d for 
                       % each generation. The ith column contains b-d for
                       % the ith generation.
                      
CI_Array = zeros(2,n); % Stores the confidence interval as a column for 
                       % each value of b-d.

Lsq_Opts = optimset('Display','off'); % Options for the curve fitter to
                                      % surpress its display each
                                      % iteration. 

% Next we create a cell array containing the equations that we want to fit 
% for each generation:

eqnarray = cell(1,n); % Create an empty cell array of n columns.

for i = 1:n
    
    Growth_eqn = @(c, t_data)A_data(1,i)*exp(c*t_data); % Define the 
                                                        % equation of
                                                        % growth for each
                                                        % generation. We
                                                        % will fit c=b-d.
    
    eqnarray{1,i} = Growth_eqn; % Add to the ith cell, the curve to be
                                % fitted for the ith generation.
    
end
                                      
% Now that we have the basic set up, we will create a loop to fit the data
% for each generation, store the parameter values and their confidence
% intervals.

for i = 1:n
                     
    c0 = 1; % We initially guess b-d=1 since we will have positive growth 
            % in each individual generation.
           
    [x,~,residual,~,~,~,jacobian] = lsqcurvefit(eqnarray(1,i), c0, t_data, A_data(:,i)',[],[],Lsq_Opts); % Fit the curve and surpress fitter output.
    
    Para_Mat(1,i) = x; % Add to the ith column, b-d found for the ith
                       % generation.
    
    CI_Array(:,i) = nlparci(x,residual,'jacobian',jacobian); % ith column
                                                             % contains the
                                                             % CI for the
                                                             % ith fitted
                                                             % parameter.
    
end

% Next we plot the synthetic data and fitted curve against time for each
% generation.

% Note that for the legend we only desire labels for the fitted curves and
% not the data.

hold on
grid on
grid minor

Legendcandidates = zeros(1,n);  % Set up vectors to store data for the
Legendinfo = cell(1,n);         % legend.   
                               

for i = 1:n
    
    plot(tspan, A_data(:,i), 'k.'); % Plot the data for the ith generation
                                    % as black dots.
    
    p = plot(tspan, eqnarray{1,i}(Para_Mat(:,i),tspan - (i-1)*T), 'linewidth', 3); % Plots the fitted curve for each generation.
    
    Legendcandidates(i) = p; % Document only the fitted curves, for which 
                             % we desire labels.
    
    Legendinfo{i} = ['A' num2str(i)]; % Create a label A'i' for the ith
                                      % fitted curve.
   
    tspan = tspan + T;  % Plot the (i+1)th generation T time units ahead of
                        % the ith generation.
    
    
end

legend(Legendcandidates,Legendinfo); % Create the legend for the fitted
                                     % curves only.
                                     
xlabel('time');
ylabel('Number of aphids'); % Label the axis appropriately.


Para_Mat % Output the parameters that were found and their respective                          
CI_Array % confidence intervals.

end



function dAdt = growthode(t,A,b,d)

% This function defines the ODE that governs the growth of a single
% generation of aphids on a leaf disc. The equation is essentially the same
% for all generations of aphids, the important factors which may differ are
% the birth rate of aphids, b, and the death rate of aphids, d.

dAdt = b*A - d*A;

end