function experiment_model_fitter_2_alternate

% (c) Ali Tariq, 2018.

clc
close all
clear all

n = 7;   % The number of generations the simulation is run for.

T = 10;   % The length of time a generation is left to grow for. We keep it
          % the same for all generations.

tspan = linspace(0,T);  % Time interval over which a solution is plotted.
                        % This creates 100 evenly spaced points between 0
                        % and T.

A_mat = zeros(length(tspan),n); % This is a zero matrix which will accept
                                % the output values for each generation of
                                % aphids over the time interval. Each
                                % column will store the data for each
                                % generation.

% We will set up a loop to solve the equation for the n generations of
% aphids we want to run the simulation for. However the first generation
% of aphids will be unaffected by the transgenic plants, meaning their
% birth and death rates will vary from subseqent generations. Therefore, we
% will run the simulation for the first generation spearately and create a
% loop for generations 2 to n.

A0 = 15;     % The initial number of aphids in the first generation.
b0 = 0.15;    % The birth rate for the first generation.
d0 = 0.01;      % The death rate for the first generation.

[~,Sol] = ode45(@(t,A)growthode(t,A,b0,d0), tspan, A0);  % Call solver.

A_mat(:,1) = Sol; % Replace the first column of A_mat with the data output
                  % from solving the equation for the first generation.

% Now we run a loop to add to A_mat the equivalent data for the remaining
% generations (2 to n). We assune that for generation 2 onwards the
% birth and death rate remain the same. Should you want them to change for
% each generation you must define this directly in the loop.

b = 0.05;   % Birth rate of generation 2 to n.
d = 0.01;    % Death rate of generation 2 to n.

A_ini = ( (b0*A0) / (b0-d0) )*((exp((b0-d0)*T)) - 1); % Initial value for 
                                                      % subsequent
                                                      % generations.
                                                      
                                                      

for i = 2:n  % the loop runs from generation 2 to n. i is the iterator.
    
    [~,Sol] = ode45(@(t,A)growthode(t,A,b,d), tspan, A_ini); % Call solver.
    
    A_mat(:,i) = Sol; % Append the ith solution to the ith column of A_mat.
    
    A_ini = ( (b*A_ini) / (b-d) )*((exp((b-d)*T)) - 1); % Redefine the
                                                        % initial
                                                        % condition.
                                                        
end

% At this point we should now have a 100xn matrix, A_mat, whose ith column
% contains the population growth data for the ith generation over the
% growth period of time length, T. 

% Now we add some normally distributed noise to A_mat to cloud the data.
% Then we will use Matlab's curve fitter to try to obtain the value of b-d.

M_noise = 0.3*randn(length(tspan),n); % Create a matrix of normally 
                                      % distributed noise.

Growth_data = abs(A_mat + M_noise); % Add the noise to the solutions of the 
                                    % ODE to create our synthetic data. We 
                                    % take the absolute value since data 
                                    % should be positive.
                               
% Growth_data now contains 100 data measurements for each generation. This 
% is unrealistic, so we select a few meaasurements for each generation to
% try and mimic real life:

A_data = [Growth_data(1,:); Growth_data(10,:); Growth_data(20,:); Growth_data(30,:); Growth_data(40,:); Growth_data(50,:); Growth_data(60,:); Growth_data(70,:); Growth_data(80,:); Growth_data(90,:); Growth_data(100,:)];

t_data = [tspan(1) tspan(10) tspan(20) tspan(30) tspan(40) tspan(50) tspan(60) tspan(70) tspan(80) tspan(90) tspan(100)];
                               
% A_data and t_data contain the measurements we pick and the time at
% which the measurement was taken, respectively.                               
                               
% Next we create a cell array containing the equations that we want to fit 
% for each generation:

eqnarray = cell(1,n); % Create an empty cell array of n columns.

for i = 1:n
    
    Growth_eqn = @(c, x)A_data(1,i)*exp(c*x); % Define the 
                                              % equation of
                                              % growth for each
                                              % generation. We
                                              % will fit c=b-d.
    
    eqnarray{1,i} = Growth_eqn; % Add to the ith cell, the curve to be
                                % fitted for the ith generation.
    
end
                                      
% Now that we have the basic set up, we will create a loop to fit the data
% for each generation, store the parameter values and their confidence
% intervals.

hold on
grid on
grid minor

Legendcandidates = zeros(1,n); % Create vectors to store the information
Legendinfo = cell(1,n);        % need to create the legend.

for i = 1:n
                     
    [fitted_curve, GOFstats] = fit(t_data' - (i-1)*T, A_data(:,i), eqnarray{1,i},'StartPoint', 1) 
                                % Fit the curve to the data for the ith
                                % generation with initial guess b-d=1. Also
                                % obtain goodness of fit statistics.
                          
    
    plot(t_data, A_data(:,i), 'kx', 'MarkerSize', 10); % Plot the synthetic
                                                       % data as black
                                                       % crosses.
    
    p = plot(tspan, fitted_curve(tspan - (i-1)*T), 'linewidth', 2); 
                          % plot the fitted curve for the ith generation.
                          
    Legendcandidates(i) = p; % Document only the fitted curves, for which 
                             % we desire labels.
    
    Legendinfo{i} = ['A' num2str(i)]; % Create a label A'i' for the ith
                                      % fitted curve.

    tspan = tspan + T;   % Shift the time interval to shift the i+1 plot 
    t_data = t_data + T; % to the right by T units.
        
end 

legend(Legendcandidates, Legendinfo); % Create the legend for the fitted
                                      % curves only.

xlabel('time');
ylabel('Number of Aphids'); % Label axis appropriately.
end



function dAdt = growthode(t,A,b,d)

% This function defines the ODE that governs the growth of a single
% generation of aphids on a leaf disc. The equation is essentially the same
% for all generations of aphids, the important factors which may differ are
% the birth rate of aphids, b, and the death rate of aphids, d.

dAdt = b*A - d*A;

end